==> type61chk-0.0.1.tar.gz: type61chk/src/curl.c <==
/*
 * Copyright (c) 2015 Vasa Consulting LLC
 *
 * Portions Copyright (C) 1998 - 2014,
 *   Daniel Stenberg, <daniel@haxx.se>, et al.
 *
 * This file is part of TYPE61CHK.
 *
 * TYPE61CHK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TYPE61CHK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TYPE61CHK.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Much of the code in this file is derived from cURL's SMTP example
 */

#include <sysexits.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include "support.h"

struct upload_status {
	size_t	lines_read;
};

extern char *smtp_username, *smtp_password, *smtp_server;

static CURL				   *ch = NULL;
static struct curl_slist   *ch_recp = NULL;
static char				  **cb_txt = NULL;
static struct upload_status	cb_ctx;
static char				   *email_from = NULL;

static size_t curl_read_cb(void *ptr, size_t size, size_t nmemb, void *userp);

_Bool
init_curl(char *from)
{
	_Bool	retval = false;

	ch = curl_easy_init();

	if (ch != NULL)
		retval = true;

	if (from != NULL)
		email_from = strndup(from, strnlen(from, 254));

	cb_ctx.lines_read = 0;

    curl_easy_setopt(ch, CURLOPT_USERNAME, smtp_username);
    curl_easy_setopt(ch, CURLOPT_PASSWORD, smtp_password);
    curl_easy_setopt(ch, CURLOPT_URL, smtp_server);
    curl_easy_setopt(ch, CURLOPT_USE_SSL, (long) CURLUSESSL_ALL);
    curl_easy_setopt(ch, CURLOPT_MAIL_FROM, email_from);
    curl_easy_setopt(ch, CURLOPT_READFUNCTION, curl_read_cb);
    curl_easy_setopt(ch, CURLOPT_READDATA, &cb_ctx);
    curl_easy_setopt(ch, CURLOPT_UPLOAD, 1L);
	curl_easy_setopt(ch, CURLOPT_SSL_VERIFYPEER, 0L);
	curl_easy_setopt(ch, CURLOPT_SSL_VERIFYHOST, 0L);

	return retval;
}

void
add_recp_to_curl(char *recp)
{
    ch_recp = curl_slist_append(ch_recp, recp);
    curl_easy_setopt(ch, CURLOPT_MAIL_RCPT, ch_recp);
}

int
add_msg_to_curl(char *msg, char *sender)
{
	time_t		now;
	struct tm	tm;
	char	  **start = NULL;
	char	   *local_msg = NULL;
	char	   *to = NULL;
	char		to_fmt[] = "To: %s\r\n";
	size_t		to_sz = 0;
	size_t		to_written = 0;

	char		from1[]	= "From: ";
	char		from2[]	= " (type61chk)\r\n";
	size_t		fromlen = strlen(from1) + strlen(from2) + strlen(smtp_username);
	char		subj[]	= "Subject: OPENPGPKEY results\r\n";
	char		newl[]	= "\r\n";

	to_sz = strlen(to_fmt) + strlen(sender);
	to = malloc(to_sz + 1);

	if (to == NULL)
		return EX_DATAERR;

	to[to_sz] = '\0';
	to_written = (size_t) snprintf(to, to_sz, to_fmt, sender);

	if (to_written >= to_sz)
		return EX_DATAERR;

	local_msg = malloc(strlen(msg) + 2 /* CRLF */ + 1);

	if (local_msg == NULL)
		return EX_OSERR;

	memcpy(local_msg, msg, strlen(msg));

	local_msg[strlen(msg)]		= '\r';
	local_msg[strlen(msg)+1]	= '\n';
	local_msg[strlen(msg)+2]	= '\0';

	time(&now);
	tm = *(localtime(&now));

	cb_txt = malloc(sizeof(char *) * 7);
	start = cb_txt;

	/* Date: */
	*cb_txt = malloc(sizeof(char) * 128);
	if (cb_txt == NULL)
		return EX_OSERR;

	strftime(*cb_txt,
			 128,
			 "Date: %a, %e %b %G %H:%M:%S %z\r\n",
			 &tm
			);
	cb_txt++;

	/* From: */
	*cb_txt = malloc(fromlen + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy(*cb_txt, from1, strlen(from1));
	memcpy(*cb_txt + strlen(from1), smtp_username, strlen(smtp_username));
	memcpy(*cb_txt + strlen(from1) + strlen(smtp_username), from2, strlen(from2));
	(*cb_txt)[fromlen] = '\0';
	cb_txt++;

	/* To: */
	*cb_txt = malloc(strlen(to) + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy(*cb_txt, to, strlen(to));
	(*cb_txt)[strlen(to)] = '\0';
	cb_txt++;

	/* Subject: */
	*cb_txt = malloc(strlen(subj) + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy(*cb_txt, subj, strlen(subj));
	(*cb_txt)[strlen(subj)] = '\0';
	cb_txt++;

	/* \r\n */
	*cb_txt = malloc(strlen(newl) + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy((*cb_txt), newl, strlen(newl));
	(*cb_txt)[strlen(newl)] = '\0';
	cb_txt++;

	/* Message body */
	*cb_txt = local_msg;
	cb_txt++;

	*cb_txt = NULL;
	cb_txt = start;

	free(to);

	return EX_OK;
}

void
free_curl()
{
    curl_slist_free_all(ch_recp);
    curl_easy_cleanup(ch);

	if (cb_txt != NULL)
	{
		char **start;
		for (start = cb_txt; *start != NULL; start++)
			free(*start);

		free(cb_txt);
	 }

	if (email_from != NULL)
		free(email_from);
}

_Bool
send_email_via_curl()
{
	_Bool		retval = false;
	CURLcode	res;

    res = curl_easy_perform(ch);

	if (res == CURLE_OK)
		retval = true;
	else
		fprintf(stderr, "curl_easy_perform() failed: %s\n",
				curl_easy_strerror(res));

	return retval;
}

static size_t
curl_read_cb(void *ptr, size_t size, size_t nmemb, void *userp)
{
	char  **pos = cb_txt;
	size_t	i;

	struct upload_status *ctx = (struct upload_status *) userp;
	const char *data;

	for (i = 0; i < ctx->lines_read; i++)
		pos++;

	if ((size == 0) || (nmemb == 0) || ((size*nmemb) < 1))
		return 0;

	data = *(pos);

	if (data)
	{
		size_t len = strlen(data);
		memcpy(ptr, data, len);
		ctx->lines_read++;

		return len;
	}

	return 0;
}

==> type61chk-0.1.0.tar.gz: type61chk/src/curl.c <==
/*
 * Copyright (c) 2015 Vasa Consulting LLC
 *
 * Portions Copyright (C) 1998 - 2014,
 *   Daniel Stenberg, <daniel@haxx.se>, et al.
 *
 * This file is part of TYPE61CHK.
 *
 * TYPE61CHK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TYPE61CHK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TYPE61CHK.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Much of the code in this file is derived from cURL's SMTP example
 */

#include <sysexits.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>
#include "support.h"

struct upload_status {
	size_t	lines_read;
};

extern char *smtp_username, *smtp_password, *smtp_server;

static CURL				   *ch = NULL;
static struct curl_slist   *ch_recp = NULL;
static char				  **cb_txt = NULL;
static struct upload_status	cb_ctx;
static char				   *email_from = NULL;

static size_t curl_read_cb(void *ptr, size_t size, size_t nmemb, void *userp);

_Bool
init_curl(char *from)
{
	_Bool	retval = false;

	ch = curl_easy_init();

	if (ch != NULL)
		retval = true;

	if (from != NULL)
		email_from = strndup(from, strnlen(from, 254));

	cb_ctx.lines_read = 0;

	curl_easy_setopt(ch, CURLOPT_USERNAME, smtp_username);
	curl_easy_setopt(ch, CURLOPT_PASSWORD, smtp_password);
	curl_easy_setopt(ch, CURLOPT_URL, smtp_server);
	curl_easy_setopt(ch, CURLOPT_USE_SSL, (long) CURLUSESSL_ALL);
	curl_easy_setopt(ch, CURLOPT_MAIL_FROM, email_from);
	curl_easy_setopt(ch, CURLOPT_READFUNCTION, curl_read_cb);
	curl_easy_setopt(ch, CURLOPT_READDATA, &cb_ctx);
	curl_easy_setopt(ch, CURLOPT_UPLOAD, 1L);
	curl_easy_setopt(ch, CURLOPT_SSL_VERIFYPEER, 0L);
	curl_easy_setopt(ch, CURLOPT_SSL_VERIFYHOST, 0L);

	return retval;
}

_Bool
add_recp_to_curl(char *recp)
{
	struct curl_slist *new = NULL;
	new = curl_slist_append(ch_recp, recp);
	if (new == NULL)
		return false;
	ch_recp = new;
	curl_easy_setopt(ch, CURLOPT_MAIL_RCPT, ch_recp);
	return true;
}

int
add_msg_to_curl(char *msg, char *sender)
{
	time_t		now;
	struct tm	tm;
	char	  **start = NULL;
	char	   *local_msg = NULL;
	char	   *to = NULL;
	char		to_fmt[] = "To: %s\r\n";
	size_t		to_sz = 0;
	size_t		to_written = 0;

	char		from1[]	= "From: ";
	char		from2[]	= " (type61chk)\r\n";
	size_t		fromlen = strlen(from1) + strlen(from2) + strlen(smtp_username);
	char		subj[]	= "Subject: OPENPGPKEY results\r\n";
	char		newl[]	= "\r\n";

	to_sz = strlen(to_fmt) + strlen(sender);
	to = malloc(to_sz + 1);

	if (to == NULL)
		return EX_DATAERR;

	to[to_sz] = '\0';
	to_written = (size_t) snprintf(to, to_sz, to_fmt, sender);

	if (to_written >= to_sz)
		return EX_DATAERR;

	local_msg = malloc(strlen(msg) + 2 /* CRLF */ + 1);

	if (local_msg == NULL)
		return EX_OSERR;

	memcpy(local_msg, msg, strlen(msg));

	local_msg[strlen(msg)]		= '\r';
	local_msg[strlen(msg)+1]	= '\n';
	local_msg[strlen(msg)+2]	= '\0';

	time(&now);
	tm = *(localtime(&now));

	cb_txt = malloc(sizeof(char *) * 7);
	start = cb_txt;

	/* Date: */
	*cb_txt = malloc(sizeof(char) * 128);
	if (cb_txt == NULL)
		return EX_OSERR;

	strftime(*cb_txt,
			 128,
			 "Date: %a, %e %b %G %H:%M:%S %z\r\n",
			 &tm
			);
	cb_txt++;

	/* From: */
	*cb_txt = malloc(fromlen + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy(*cb_txt, from1, strlen(from1));
	memcpy(*cb_txt + strlen(from1), smtp_username, strlen(smtp_username));
	memcpy(*cb_txt + strlen(from1) + strlen(smtp_username), from2, strlen(from2));
	(*cb_txt)[fromlen] = '\0';
	cb_txt++;

	/* To: */
	*cb_txt = malloc(strlen(to) + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy(*cb_txt, to, strlen(to));
	(*cb_txt)[strlen(to)] = '\0';
	cb_txt++;

	/* Subject: */
	*cb_txt = malloc(strlen(subj) + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy(*cb_txt, subj, strlen(subj));
	(*cb_txt)[strlen(subj)] = '\0';
	cb_txt++;

	/* \r\n */
	*cb_txt = malloc(strlen(newl) + 1);
	if (cb_txt == NULL)
		return EX_OSERR;
	memcpy((*cb_txt), newl, strlen(newl));
	(*cb_txt)[strlen(newl)] = '\0';
	cb_txt++;

	/* Message body */
	*cb_txt = local_msg;
	cb_txt++;

	*cb_txt = NULL;
	cb_txt = start;

	free(to);

	return EX_OK;
}

void
free_curl()
{
	curl_slist_free_all(ch_recp);
	curl_easy_cleanup(ch);

	if (cb_txt != NULL)
	{
		char **start;
		for (start = cb_txt; *start != NULL; start++)
			free(*start);

		free(cb_txt);
	 }

	if (email_from != NULL)
		free(email_from);
}

_Bool
send_email_via_curl()
{
	_Bool		retval = false;
	CURLcode	res;

	res = curl_easy_perform(ch);

	if (res == CURLE_OK)
		retval = true;
	else
		fprintf(stderr, "curl_easy_perform() failed: %s\n",
				curl_easy_strerror(res));

	return retval;
}

static size_t
curl_read_cb(void *ptr, size_t size, size_t nmemb, void *userp)
{
	char  **pos = cb_txt;
	size_t	i;

	struct upload_status *ctx = (struct upload_status *) userp;
	const char *data;

	for (i = 0; i < ctx->lines_read; i++)
		pos++;

	if ((size == 0) || (nmemb == 0) || ((size*nmemb) < 1))
		return 0;

	data = *(pos);

	if (data)
	{
		size_t len = strlen(data);
		memcpy(ptr, data, len);
		ctx->lines_read++;

		return len;
	}

	return 0;
}
